@@ -29,45 +29,12 @@
 
 /* These are global */
 gnutls_certificate_credentials cred;
-
-gnutls_session initialize_tls_session()
-{
-   gnutls_session session;
-   const int cert_type_priority[2] = { GNUTLS_CRT_OPENPGP, 0 };
-
-   gnutls_init(&session, GNUTLS_SERVER);
-
-   /* avoid calling all the priority functions, since the defaults
-    * are adequate.
-    */
-   gnutls_set_default_priority( session);
-   gnutls_certificate_type_set_priority(session, cert_type_priority);
-
-   gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, cred);
-
-   /* request client certificate.
-    */
-   gnutls_certificate_server_set_request( session, GNUTLS_CERT_REQUEST);
-
-   gnutls_dh_set_prime_bits( session, DH_BITS);
-
-   return session;
-}
-
+const int cert_type_priority[2] = { GNUTLS_CRT_OPENPGP, 0 };
 gnutls_dh_params dh_params;
 
-static int generate_dh_params(void) {
-
-   /* Generate Diffie Hellman parameters - for use with DHE
-    * kx algorithms. These should be discarded and regenerated
-    * once a day, once a week or once a month. Depends on the
-    * security requirements.
-    */
-   gnutls_dh_params_init( &dh_params);
-   gnutls_dh_params_generate2( dh_params, DH_BITS);
-
-   return 0;
-}
+/* Defined in a previous example */
+extern int generate_dh_params( void);
+extern gnutls_session initialize_tls_session( void);
 
 int main()
 {
@@ -119,6 +86,7 @@ int main()
    client_len = sizeof(sa_cli);
    for (;;) {
       session = initialize_tls_session();
+      gnutls_certificate_type_set_priority(session, cert_type_priority);
 
       sd = accept(listen_sd, (SA *) & sa_cli, &client_len);
 
