@@ -1,5 +1,5 @@
 /*
- *      Copyright (C) 2001 Nikos Mavroyanopoulos
+ * Copyright (C) 2001,2003 Nikos Mavroyanopoulos
  *
  * This file is part of GNUTLS.
  *
@@ -20,15 +20,19 @@
 
 #include <gnutls_int.h>
 
-void* _gnutls_ext_func_send(uint16 type);
-void* _gnutls_ext_func_recv(uint16 type);
 const char *_gnutls_extension_get_name(uint16 type);
 int _gnutls_parse_extensions( gnutls_session, const opaque*, int);
-int _gnutls_gen_extensions( gnutls_session session, opaque** data);
+int _gnutls_gen_extensions( gnutls_session session, opaque* data, size_t data_size);
+
+typedef int (*ext_recv_func)( gnutls_session, const opaque*, size_t); /* recv data */
+typedef int (*ext_send_func)( gnutls_session, opaque*, size_t); /* send data */
+
+ext_send_func _gnutls_ext_func_send(uint16 type);
+ext_recv_func _gnutls_ext_func_recv(uint16 type);
 
 typedef struct {
 	const char *name;
 	uint16 type;
-	int (*gnutls_ext_func_recv)( gnutls_session, const opaque*, size_t); /* recv data */
-	int (*gnutls_ext_func_send)( gnutls_session, opaque*, size_t); /* send data */
+	ext_recv_func gnutls_ext_func_recv;
+	ext_send_func gnutls_ext_func_send;
 } gnutls_extension_entry;
