@@ -1,5 +1,6 @@
 /* pkcs5.c	Implementation of Password-Based Cryptography as per PKCS#5
- * Copyright (C) 2002  Simon Josefsson
+ * Copyright (C) 2002,2003 Simon Josefsson
+ * Copyright (C) 2004 Free Software Foundation
  *
  * This file is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,8 +18,6 @@
  *
  */
 
-/* XXX what about namespace? */
-
 #include <gnutls_int.h>
 
 #ifdef ENABLE_PKI
@@ -43,7 +42,7 @@
  *                             denotes the length in octets of the
  *                             pseudorandom function output)
  *
- *  Input:          P          password, an octet string
+ *  Input:          P          password, an octet string (ASCII or UTF-8)
  *                  S          salt, an octet string
  *                  c          iteration count, a positive integer
  *                  dkLen      intended length in octets of the derived
@@ -84,10 +83,6 @@ _gnutls_pkcs5_pbkdf2 (int PRF,
   if (dkLen == 0)
     return PKCS5_INVALID_DERIVED_KEY_LENGTH;
 
-  if ((rc=_pkcs12_check_pass( P, Plen)) < 0) {
-  	return rc;
-  }
-
   /*
    *
    *  Steps:
@@ -168,8 +163,6 @@ _gnutls_pkcs5_pbkdf2 (int PRF,
 
       for (u = 1; u <= c; u++)
 	{
-	  int Ulen;
-
 	  gcry_md_reset (prf);
 
 	  rc = gcry_md_setkey (prf, P, Plen);
@@ -178,19 +171,26 @@ _gnutls_pkcs5_pbkdf2 (int PRF,
 
 	  if (u == 1)
 	    {
-	      char tmp[4];
-	      memcpy (U, S, Slen);
-	      tmp[0] = (i & 0xff000000) >> 24;
-	      tmp[1] = (i & 0x00ff0000) >> 16;
-	      tmp[2] = (i & 0x0000ff00) >> 8;
-	      tmp[3] = (i & 0x000000ff) >> 0;
-	      memcpy (U + Slen, tmp, 4);
-	      Ulen = Slen + 4;
+	      char *tmp;
+	      size_t tmplen = Slen + 4;
+	      
+	      tmp = gnutls_alloca( tmplen);
+	      if (tmp == NULL)
+	         return PKCS5_INVALID_PRF;
+
+	      memcpy (tmp, S, Slen);
+	      tmp[Slen + 0] = (i & 0xff000000) >> 24;
+	      tmp[Slen + 1] = (i & 0x00ff0000) >> 16;
+	      tmp[Slen + 2] = (i & 0x0000ff00) >> 8;
+	      tmp[Slen + 3] = (i & 0x000000ff) >> 0;
+
+  	      gcry_md_write (prf, tmp, tmplen);
+  	      gnutls_afree(tmp);
 	    }
 	  else
-	    Ulen = hLen;
-
-	  gcry_md_write (prf, U, Ulen);
+	    {
+  	      gcry_md_write (prf, U, hLen);
+	    }
 
 	  p = gcry_md_read (prf, PRF);
 	  if (p == NULL)
