@@ -1,5 +1,6 @@
 /*
  *  Copyright (C) 2000,2001,2003 Nikos Mavroyanopoulos <nmav@hellug.gr>
+ *  Copyright (C) 2004 Free Software Foundation
  *
  *  This file is part of GNUTLS.
  *
@@ -163,12 +164,14 @@ int _gnutls_base64_encode(const uint8 * data, size_t data_size, uint8 ** result)
 }
 
 #define INCR(what, size) \
+	do { \
 	what+=size; \
 	if (what > ret) { \
 		gnutls_assert(); \
 		gnutls_free( (*result)); *result = NULL; \
 		return GNUTLS_E_INTERNAL_ERROR; \
-	}
+	} \
+	} while(0) 
 
 /* encodes data and puts the result into result (locally allocated)
  * The result_size (including the null terminator) is the return value.
@@ -313,8 +316,7 @@ int size;
   * encoding. This is the encoding used in PEM messages. This function will
   * allocate the required memory to hold the encoded data.
   *
-  * You should use the function gnutls_free() to
-  * free the returned data.
+  * You should use gnutls_free() to free the returned data.
   * 
   **/
 int gnutls_pem_base64_encode_alloc( const char* msg, const gnutls_datum *data, 
@@ -524,8 +526,7 @@ int size;
   * "-----BEGIN header" and decode only this part. Otherwise it will decode the 
   * first PEM packet found.
   *
-  * You should use the function gnutls_free() to
-  * free the returned data.
+  * You should use gnutls_free() to free the returned data.
   *
   **/
 int gnutls_pem_base64_decode_alloc( const char* header, const gnutls_datum *b64_data, 
